@@ -787,7 +787,9 @@ class TestQueryJob(_Base):
                 "location": "EU",
             },
             "schema": {"fields": [{"name": "col1", "type": "STRING"}]},
-            "totalRows": "2",
+            "totalRows": "3",
+            "rows": [{"f": [{"v": "abc"}]}],
+            "pageToken": "next-page",
         }
         job_resource = self._make_resource(started=True, location="EU")
         job_resource_done = self._make_resource(started=True, ended=True, location="EU")
@@ -799,9 +801,9 @@ class TestQueryJob(_Base):
         query_page_resource = {
             # Explicitly set totalRows to be different from the initial
             # response to test update during iteration.
-            "totalRows": "1",
+            "totalRows": "2",
             "pageToken": None,
-            "rows": [{"f": [{"v": "abc"}]}],
+            "rows": [{"f": [{"v": "def"}]}],
         }
         conn = _make_connection(
             query_resource, query_resource_done, job_resource_done, query_page_resource
@@ -812,19 +814,20 @@ class TestQueryJob(_Base):
         result = job.result()
 
         self.assertIsInstance(result, RowIterator)
-        self.assertEqual(result.total_rows, 2)
+        self.assertEqual(result.total_rows, 3)
         rows = list(result)
-        self.assertEqual(len(rows), 1)
+        self.assertEqual(len(rows), 2)
         self.assertEqual(rows[0].col1, "abc")
+        self.assertEqual(rows[1].col1, "def")
         # Test that the total_rows property has changed during iteration, based
         # on the response from tabledata.list.
-        self.assertEqual(result.total_rows, 1)
+        self.assertEqual(result.total_rows, 2)
 
         query_results_path = f"/projects/{self.PROJECT}/queries/{self.JOB_ID}"
         query_results_call = mock.call(
             method="GET",
             path=query_results_path,
-            query_params={"maxResults": 0, "location": "EU"},
+            query_params={"location": "EU"},
             timeout=None,
         )
         reload_call = mock.call(
@@ -839,6 +842,7 @@ class TestQueryJob(_Base):
             query_params={
                 "fields": _LIST_ROWS_FROM_QUERY_RESULTS_FIELDS,
                 "location": "EU",
+                "pageToken": "next-page",
             },
             timeout=None,
         )
@@ -851,7 +855,9 @@ class TestQueryJob(_Base):
             "jobComplete": True,
             "jobReference": {"projectId": self.PROJECT, "jobId": self.JOB_ID},
             "schema": {"fields": [{"name": "col1", "type": "STRING"}]},
-            "totalRows": "1",
+            "totalRows": "2",
+            "rows": [{"f": [{"v": "abc"}]}],
+            "pageToken": "next-page",
         }
         job_resource = self._make_resource(started=True, ended=True, location="EU")
         job_resource["configuration"]["query"]["destinationTable"] = {
@@ -860,9 +866,9 @@ class TestQueryJob(_Base):
             "tableId": "dest_table",
         }
         results_page_resource = {
-            "totalRows": "1",
+            "totalRows": "2",
             "pageToken": None,
-            "rows": [{"f": [{"v": "abc"}]}],
+            "rows": [{"f": [{"v": "def"}]}],
         }
         conn = _make_connection(query_resource_done, results_page_resource)
         client = _make_client(self.PROJECT, connection=conn)
@@ -871,14 +877,15 @@ class TestQueryJob(_Base):
         result = job.result()
 
         rows = list(result)
-        self.assertEqual(len(rows), 1)
+        self.assertEqual(len(rows), 2)
         self.assertEqual(rows[0].col1, "abc")
+        self.assertEqual(rows[1].col1, "def")
 
         query_results_path = f"/projects/{self.PROJECT}/queries/{self.JOB_ID}"
         query_results_call = mock.call(
             method="GET",
             path=query_results_path,
-            query_params={"maxResults": 0, "location": "EU"},
+            query_params={"location": "EU"},
             timeout=None,
         )
         query_results_page_call = mock.call(
@@ -887,6 +894,7 @@ class TestQueryJob(_Base):
             query_params={
                 "fields": _LIST_ROWS_FROM_QUERY_RESULTS_FIELDS,
                 "location": "EU",
+                "pageToken": "next-page",
             },
             timeout=None,
         )
@@ -900,6 +908,12 @@ class TestQueryJob(_Base):
             "jobReference": {"projectId": self.PROJECT, "jobId": self.JOB_ID},
             "schema": {"fields": [{"name": "col1", "type": "STRING"}]},
             "totalRows": "5",
+            # These rows are discarded because max_results is set.
+            "rows": [
+                {"f": [{"v": "xyz"}]},
+                {"f": [{"v": "uvw"}]},
+                {"f": [{"v": "rst"}]},
+            ],
         }
         query_page_resource = {
             "totalRows": "5",
@@ -925,6 +939,7 @@ class TestQueryJob(_Base):
         rows = list(result)
 
         self.assertEqual(len(rows), 3)
+        self.assertEqual(rows[0].col1, "abc")
         self.assertEqual(len(connection.api_request.call_args_list), 2)
         query_page_request = connection.api_request.call_args_list[1]
         self.assertEqual(
@@ -979,7 +994,7 @@ class TestQueryJob(_Base):
         query_results_call = mock.call(
             method="GET",
             path=f"/projects/{self.PROJECT}/queries/{self.JOB_ID}",
-            query_params={"maxResults": 0, "location": "asia-northeast1"},
+            query_params={"location": "asia-northeast1"},
             timeout=None,
         )
         reload_call = mock.call(
@@ -1079,6 +1094,12 @@ class TestQueryJob(_Base):
             "jobReference": {"projectId": self.PROJECT, "jobId": self.JOB_ID},
             "schema": {"fields": [{"name": "col1", "type": "STRING"}]},
             "totalRows": "4",
+            # These rows are discarded because page_size is set.
+            "rows": [
+                {"f": [{"v": "xyz"}]},
+                {"f": [{"v": "uvw"}]},
+                {"f": [{"v": "rst"}]},
+            ],
         }
         job_resource = self._make_resource(started=True, ended=True, location="US")
         q_config = job_resource["configuration"]["query"]
@@ -1109,6 +1130,7 @@ class TestQueryJob(_Base):
         # Assert
         actual_rows = list(result)
         self.assertEqual(len(actual_rows), 4)
+        self.assertEqual(actual_rows[0].col1, "row1")
 
         query_results_path = f"/projects/{self.PROJECT}/queries/{self.JOB_ID}"
         query_page_1_call = mock.call(
@@ -1142,6 +1164,12 @@ class TestQueryJob(_Base):
             "jobReference": {"projectId": self.PROJECT, "jobId": self.JOB_ID},
             "schema": {"fields": [{"name": "col1", "type": "STRING"}]},
             "totalRows": "5",
+            # These rows are discarded because start_index is set.
+            "rows": [
+                {"f": [{"v": "xyz"}]},
+                {"f": [{"v": "uvw"}]},
+                {"f": [{"v": "rst"}]},
+            ],
         }
         tabledata_resource = {
             "totalRows": "5",
@@ -1168,6 +1196,7 @@ class TestQueryJob(_Base):
         rows = list(result)
 
         self.assertEqual(len(rows), 4)
+        self.assertEqual(rows[0].col1, "abc")
         self.assertEqual(len(connection.api_request.call_args_list), 2)
         tabledata_list_request = connection.api_request.call_args_list[1]
         self.assertEqual(
